
;|
;|  Titel....: 2-Dimensionale Vektoren
;|  Datei....: Vector2D.pbi
;|  Datum....: 09.08.2010
;|  Inhalt...: * Macros und Proceduren fr 2-Dimensionale Vektorberechungen
;|_____________________________________________________________________________






;  Vorrausgesetzte Includes
;
 
 
 



;  Konstanten
;






;  Strukturen
;



 ; Structure eines 2D-Vektors
 Structure Vector2D
  x.f
  y.f
 EndStructure






;  Arrays und LinkedLists
;






;  Proceduren und Macros
;



 ; Setzt die Werte (x,y) in den Vektor ein 
 Macro SetVector2D(Vector2D, xValue, yValue)
  Vector2D\x = xValue
  Vector2D\y = yValue
 EndMacro
 
 ; Gibt den Vektor als String aus
 Macro GetVector2D(Vector2D)
  "( "+StrF(Vector2D\x)+" | "+StrF(Vector2D\y)+" )"
 EndMacro
 
 

 
 ; Umrechnung von Vektor zu Winkel
 Macro Vector2DToAngle(Vector2D, Angle)
  Angle = ATan2(Vector2D\x, Vector2D\y) 
 EndMacro
 
 ; Umrechnung von Winkel zu normiertem Vektor
 Macro AngleToVector2D(Angle, Vector2D)
  Vector2D\x = Cos(Angle)
  Vector2D\y = Sin(Angle)
 EndMacro
 
 
 
 ; Gibt die quadratische Lnge des Vektors zurck
 Macro Vector2DQuaLength(Vector2D)
  ( Vector2D\x*Vector2D\x + Vector2D\y*Vector2D\y )
 EndMacro
  
 ; Gibt die Lnge des Vektors zurck
 Macro Vector2DLength(Vector2D)
  Sqr(Vector2DQuaLength(Vector2D))
 EndMacro
 

 
 
 ; Berechnet den normierten Vektor zu einem Vektor
 Procedure Vector2DNorm(*Vector2D.Vector2D, *NormVector2D.Vector2D)
  Protected Length.f = Vector2DLength(*Vector2D)
  If Length
   *NormVector2D\x = *Vector2D\x / Length
   *NormVector2D\y = *Vector2D\y / Length
  Else
   *NormVector2D\x = 0
   *NormVector2D\y = 0
  EndIf
 EndProcedure


 
 ; Gibt das Skalarprodukt zweier Vektoren zurck
 Macro Vector2DScalar(Vector2D_1, Vector2D_2)
  ( Vector2D_1\x * Vector2D_2\x + Vector2D_1\y * Vector2D_2\y )
 EndMacro
 
 
 
 ; Gibt das Kreuzprodukt zweier Vektoren zurck
 Macro Vector2DCross(Vector2D_1, Vector2D_2)
  ( Vector2D_1\x * Vector2D_2\y - Vector2D_1\y * Vector2D_2\x )
 EndMacro
 
 
 
 ; Gibt den Winkel zwischen zwei Vektoren zurck
 Macro Vector2DAngle(Vector2D_1, Vector2D_2)
  ACos( Vector2DScalar(Vector2D_1, Vector2D_2) / (Vector2DLength(Vector2D_1)*Vector2DLength(Vector2D_2)) )
 EndMacro
  

 
 ; Berechnet die Summe zweier Vektor
 Macro Vector2DAdd(Vector2D_1, Vector2D_2, ResultVector2D)
  ResultVector2D\x = Vector2D_1\x + Vector2D_2\x
  ResultVector2D\y = Vector2D_1\y + Vector2D_2\y
 EndMacro
 
 ; Berechnet die Differenz zweier Vektor
 Macro Vector2DSub(Vector2D_1, Vector2D_2, ResultVector2D)
  ResultVector2D\x = Vector2D_1\x - Vector2D_2\x
  ResultVector2D\y = Vector2D_1\y - Vector2D_2\y
 EndMacro
 
 ; Multipliziert ein Vektor mit einem Faktor
 Macro Vector2DMul(Vector2D, Factor, ResultVector2D)
  ResultVector2D\x = (Factor) * Vector2D\x
  ResultVector2D\y = (Factor) * Vector2D\y
 EndMacro
 
 
 
 ; Rotiert einen Vektor (relativ)
 Macro Vector2DRotate(Vector2D, Angle, ResultVector2D)
  ResultVector2D\x = Cos(Angle)*Vector2D\x - Sin(Angle)*Vector2D\y
  ResultVector2D\y = Sin(Angle)*Vector2D\x + Cos(Angle)*Vector2D\y
 EndMacro 
 
 
 
 ; Addiert zu einem Vektor einen anderen hinzu
 Macro MoveVector2D(Vector2D, MoveVector2D)
  Vector2D\x + MoveVector2D\x
  Vector2D\y + MoveVector2D\y
 EndMacro
 
 ; ndert die Lnge des Vektor um einen Faktor
 Macro RaiseVector2D(Vector2D, Factor)
  Vector2D\x * (Factor)
  Vector2D\y * (Factor)
 EndMacro
 
 ; ndert den Vektor auf die angegebene Lnge
 Macro LengthVector2D(Vector2D, Length)
  NormVector2D(Vector2D)
  RaiseVector2D(Vector2D, Length)
 EndMacro
  
 
 
 ; Normiert einen Vektor
 Procedure NormVector2D(*Vector2D.Vector2D)
  Protected Length.f = Vector2DLength(*Vector2D)
  If Length
   *Vector2D\x / Length
   *Vector2D\y / Length
  Else
   *Vector2D\x = 0
   *Vector2D\y = 0
  EndIf
 EndProcedure
 
 
 
 ; Rotiert einen Vektor (relativ)
 Procedure RotateVector2D(*Vector2D.Vector2D, Angle.f)
  Protected x.f = *Vector2D\x, y.f = *Vector2D\y
  Protected Sin.f = Sin(Angle), Cos.f = Cos(Angle)
  *Vector2D\x = Cos*x - Sin*y
  *Vector2D\y = Sin*x + Cos*y
 EndProcedure
 
 ; ndert den Vektor auf angegebenen Winkel
 Procedure AngleVector2D(*Vector2D.Vector2D, Angle.f)
  Protected Length.f = Vector2DLength(*Vector2D)
  *Vector2D\x = Cos(Angle)*Length
  *Vector2D\y = Sin(Angle)*Length
 EndProcedure
  
   
 ; Kopiert einen Vektor
 Macro CopyVector2D(SourceVector2D, NewVector2D)
  NewVector2D\x = SourceVector2D\x
  NewVector2D\y = SourceVector2D\y
 EndMacro


 
 ; Vergleicht zwei Vektoren auf gleichheit
 Procedure CompareVector2D(*Vector2D_1.Vector2D, *Vector2D_2.Vector2D)
  If *Vector2D_1\x = *Vector2D_2\x And *Vector2D_1\y = *Vector2D_2\y
   ProcedureReturn #True
  EndIf
 EndProcedure


 
 ; Tauscht zwei Vektoren miteinander
 Macro SwapVector2D(Vector2D_1, Vector2D_2)
  Swap Vector2D_1\x , Vector2D_2\x
  Swap Vector2D_1\y , Vector2D_2\y
 EndMacro


   
 
 ; Gibt den quadratischen Abstand zwischen zwei Vektoren zurck
 Macro QuaDistance2D(Vector2D_1, Vector2D_2)
  ( (Vector2D_1\x-Vector2D_2\x)*(Vector2D_1\x-Vector2D_2\x)+(Vector2D_1\y-Vector2D_2\y)*(Vector2D_1\y-Vector2D_2\y) )
 EndMacro
 
 ; Gibt den Abstand zwischen zwei Vektoren zurck
 Macro Distance2D(Vector2D_1, Vector2D_2)
  Sqr( QuaDistance2D(Vector2D_1, Vector2D_2) )
 EndMacro

 
 
 ; Stellt einen Vektor an Position (x,y) dar
 Macro DrawVector2D(Vector2D, xPosition, yPosition, Color=#PB_Default)
  If Color > #PB_Default
   LineXY(xPosition, yPosition, xPosition+Vector2D\x, yPosition+Vector2D\y, Color)
  Else
   LineXY(xPosition, yPosition, xPosition+Vector2D\x, yPosition+Vector2D\y)
  EndIf
 EndMacro
 
 
 
 ; Stellt einen Punkte mit dem Radius (r) dar
 Procedure DrawPoint2D(*Vector2D.Vector2D, r=0, Color=#PB_Default)
  If r
   If Color > #PB_Default
    Circle(*Vector2D\x, *Vector2D\y, r, Color)
   Else
    Circle(*Vector2D\x, *Vector2D\y, r)
   EndIf
  Else
   If Color > #PB_Default
    Plot(*Vector2D\x, *Vector2D\y, Color)
   Else
    Plot(*Vector2D\x, *Vector2D\y)
   EndIf   
  EndIf
 EndProcedure
 
 

; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 75
; FirstLine = 73